@@ -1,16 +1,17 @@
 #!/usr/bin/python -O
+# Copyright 1999-2006 Gentoo Foundation
+# Distributed under the terms of the GNU General Public License v2
+# $Id: md5check.py 6930 2007-06-22 01:53:50Z zmedico $
 
-import os,sys,string
-os.environ["PORTAGE_CALLER"]="mirror"
+import os,sys
 os.environ["FEATURES"]="mirror cvs"
-sys.path = ["/usr/lib/portage/pym"]+sys.path
-
-import portage
-from threading import *
-from output import red,green,blue,bold
-from random import shuffle
-from time import sleep
-
+try:
+	import portage
+except ImportError:
+	from os import path as osp
+	sys.path.insert(0, osp.join(osp.dirname(osp.dirname(osp.realpath(__file__))), "pym"))
+	import portage
+import portage_util
 
 def cstrip(mystr,mychars):
 	newstr = ""
@@ -29,10 +30,10 @@ for mycp in portage.db["/"]["porttree"].dbapi.cp_all():
 hugelist.sort()
 
 for mycpv in hugelist:
-	pv = string.split(mycpv, "/")[-1]
+	pv = mycpv.split("/")[-1]
 
 	newuri = portage.db["/"]["porttree"].dbapi.aux_get(mycpv,["SRC_URI"])[0]
-	newuri = string.split(newuri)
+	newuri = newuri.split()
 
 	digestpath = portage.db["/"]["porttree"].dbapi.findname(mycpv)
 	digestpath = os.path.dirname(digestpath)+"/files/digest-"+pv
@@ -67,16 +68,16 @@ for mycpv in hugelist:
 		else:
 			continue
 		
-		if mybn not in md5sums.keys():
-			portage.writemsg("Missing md5sum: %s in %s\n" % (mybn, mycpv))
+		if mybn not in md5sums:
+			portage_util.writemsg("Missing md5sum: %s in %s\n" % (mybn, mycpv))
 		else:
-			if mybn in md5_list.keys():
-				if (md5_list[mybn][0] != md5sums[mybn][0]) or \
-				   (md5_list[mybn][1] != md5sums[mybn][1]):
+			if mybn in md5_list:
+				if (md5_list[mybn]["MD5"]  != md5sums[mybn]["MD5"]) or \
+				   (md5_list[mybn]["size"] != md5sums[mybn]["size"]):
 
 					# This associates teh md5 with each file. [md5/size]
-					md5joins = string.split(md5_list[mybn][2],",")
-					md5joins = string.join(md5joins," ["+md5_list[mybn][0]+"/"+md5_list[mybn][1]+"],")
+					md5joins = md5_list[mybn][2].split(",")
+					md5joins = (" ["+md5_list[mybn][0]+"/"+md5_list[mybn][1]+"],").join(md5joins)
 					md5joins += " ["+md5_list[mybn][0]+"/"+md5_list[mybn][1]+"]"
 
 					portage.writemsg("Colliding md5: %s of %s [%s/%s] and %s\n" % (mybn,mycpv,md5sums[mybn][0],md5sums[mybn][1],md5joins))
@@ -88,12 +89,12 @@ for mycpv in hugelist:
 			del md5sums[mybn]
 		
 	#portage.writemsg(str(bn_list)+"\n")
-	for x in md5sums.keys():
+	for x in md5sums:
 		if x not in bn_list:
 			portage.writemsg("Extra md5sum: %s in %s\n" % (x, mycpv))
 
 
 print col_list
 print
-print str(len(md5_list.keys()))+" unique distfile md5s."
+print str(len(md5_list))+" unique distfile md5s."
 print str(len(bn_list))+" unique distfile names."
