@@ -1,9 +1,12 @@
 # Copyright: 2005 Gentoo Foundation
 # Author(s): Brian Harring (ferringb@gentoo.org)
 # License: GPL2
-# $Id: util.py 1911 2005-08-25 03:44:21Z ferringb $
+# $Id: util.py 6934 2007-06-22 02:50:33Z zmedico $
 
-import cache_errors
+if not hasattr(__builtins__, "set"):
+	from sets import Set as set
+from itertools import chain
+from cache import cache_errors
 
 def mirror_cache(valid_nodes_iterable, src_cache, trg_cache, eclass_cache=None, verbose_instance=None):
 
@@ -15,8 +18,7 @@ def mirror_cache(valid_nodes_iterable, src_cache, trg_cache, eclass_cache=None,
 	else:
 		noise=verbose_instance
 
-	dead_nodes = {}
-	dead_nodes = dict.fromkeys(trg_cache.keys())
+	dead_nodes = set(trg_cache)
 	count=0
 
 	if not trg_cache.autocommits:
@@ -25,21 +27,36 @@ def mirror_cache(valid_nodes_iterable, src_cache, trg_cache, eclass_cache=None,
 	for x in valid_nodes_iterable:
 #		print "processing x=",x
 		count+=1
-		if dead_nodes.has_key(x):
-			del dead_nodes[x]
+		dead_nodes.discard(x)
 		try:	entry = src_cache[x]
 		except KeyError, e:
 			noise.missing_entry(x)
 			del e
 			continue
 		write_it = True
+		trg = None
 		try:
 			trg = trg_cache[x]
-			if long(trg["_mtime_"]) == long(entry["_mtime_"]) and eclass_cache.is_eclass_data_valid(trg["_eclasses_"]):
+			if long(trg["_mtime_"]) == long(entry["_mtime_"]) and \
+				eclass_cache.is_eclass_data_valid(trg["_eclasses_"]) and \
+				set(trg["_eclasses_"]) == set(entry["_eclasses_"]):
 				write_it = False
 		except (cache_errors.CacheError, KeyError):
 			pass
 
+		if trg and not write_it:
+			""" We don't want to skip the write unless we're really sure that
+			the existing cache is identical, so don't trust _mtime_ and
+			_eclasses_ alone."""
+			for d in (entry, trg):
+				if "EAPI" in d and d["EAPI"] in ("", "0"):
+					del d["EAPI"]
+			for k in set(chain(entry, trg)).difference(
+				("_mtime_", "_eclasses_")):
+				if trg.get(k, "") != entry.get(k, ""):
+					write_it = True
+					break
+
 		if write_it:
 			try:
 				inherited = entry.get("INHERITED", None)
@@ -79,11 +96,13 @@ def mirror_cache(valid_nodes_iterable, src_cache, trg_cache, eclass_cache=None,
 	# ok.  by this time, the trg_cache is up to date, and we have a dict
 	# with a crapload of cpv's.  we now walk the target db, removing stuff if it's in the list.
 	for key in dead_nodes:
-		try:	del trg_cache[key]
+		try:
+			del trg_cache[key]
+		except KeyError:
+			pass
 		except cache_errors.CacheError, ce:
 			noise.exception(ce)
 			del ce
-	dead_nodes.clear()
 	noise.finish()
 
 
