@@ -1,9 +1,10 @@
 # cvstree.py -- cvs tree utilities
 # Copyright 1998-2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /var/cvsroot/gentoo-src/portage/pym/cvstree.py,v 1.12 2004/10/11 04:12:02 carpaski Exp $
+# $Id: cvstree.py 6931 2007-06-22 02:09:41Z zmedico $
 
-import string,os,time,sys,re
+
+import os,time,sys,re
 from stat import *
 
 # [D]/Name/Version/Date/Flags/Tags
@@ -11,7 +12,7 @@ from stat import *
 def pathdata(entries, path):
 	"""(entries,path)
 	Returns the data(dict) for a specific file/dir at the path specified."""
-	mysplit=string.split(path,"/")
+	mysplit=path.split("/")
 	myentries=entries
 	mytarget=mysplit[-1]
 	mysplit=mysplit[:-1]
@@ -63,12 +64,12 @@ def findnew(entries,recursive=0,basedir=""):
 	if basedir and basedir[-1]!="/":
 		basedir=basedir+"/"
 	mylist=[]
-	for myfile in entries["files"].keys():
+	for myfile in entries["files"]:
 		if "cvs" in entries["files"][myfile]["status"]:
 			if "0" == entries["files"][myfile]["revision"]:
 				mylist.append(basedir+myfile)
 	if recursive:
-		for mydir in entries["dirs"].keys():
+		for mydir in entries["dirs"]:
 			mylist+=findnew(entries["dirs"][mydir],recursive,basedir+mydir)
 	return mylist
 					
@@ -80,14 +81,14 @@ def findchanged(entries,recursive=0,basedir=""):
 	if basedir and basedir[-1]!="/":
 		basedir=basedir+"/"
 	mylist=[]
-	for myfile in entries["files"].keys():
+	for myfile in entries["files"]:
 		if "cvs" in entries["files"][myfile]["status"]:
 			if "current" not in entries["files"][myfile]["status"]:
 				if "exists" in entries["files"][myfile]["status"]:
 					if entries["files"][myfile]["revision"]!="0":
 						mylist.append(basedir+myfile)
 	if recursive:
-		for mydir in entries["dirs"].keys():
+		for mydir in entries["dirs"]:
 			mylist+=findchanged(entries["dirs"][mydir],recursive,basedir+mydir)
 	return mylist
 	
@@ -99,13 +100,13 @@ def findmissing(entries,recursive=0,basedir=""):
 	if basedir and basedir[-1]!="/":
 		basedir=basedir+"/"
 	mylist=[]
-	for myfile in entries["files"].keys():
+	for myfile in entries["files"]:
 		if "cvs" in entries["files"][myfile]["status"]:
 			if "exists" not in entries["files"][myfile]["status"]:
 				if "removed" not in entries["files"][myfile]["status"]:
 					mylist.append(basedir+myfile)
 	if recursive:
-		for mydir in entries["dirs"].keys():
+		for mydir in entries["dirs"]:
 			mylist+=findmissing(entries["dirs"][mydir],recursive,basedir+mydir)
 	return mylist
 
@@ -119,11 +120,11 @@ def findunadded(entries,recursive=0,basedir=""):
 	mylist=[]
 
 	#ignore what cvs ignores.
-	for myfile in entries["files"].keys():
+	for myfile in entries["files"]:
 		if "cvs" not in entries["files"][myfile]["status"]:
 			mylist.append(basedir+myfile)
 	if recursive:
-		for mydir in entries["dirs"].keys():
+		for mydir in entries["dirs"]:
 			mylist+=findunadded(entries["dirs"][mydir],recursive,basedir+mydir)
 	return mylist
 
@@ -134,11 +135,11 @@ def findremoved(entries,recursive=0,basedir=""):
 	if basedir and basedir[-1]!="/":
 		basedir=basedir+"/"
 	mylist=[]
-	for myfile in entries["files"].keys():
+	for myfile in entries["files"]:
 		if "removed" in entries["files"][myfile]["status"]:
 			mylist.append(basedir+myfile)
 	if recursive:
-		for mydir in entries["dirs"].keys():
+		for mydir in entries["dirs"]:
 			mylist+=findremoved(entries["dirs"][mydir],recursive,basedir+mydir)
 	return mylist
 
@@ -190,7 +191,7 @@ def getentries(mydir,recursive=0):
 			continue
 		if line=="D": # End of entries file
 			break
-		mysplit=string.split(line, "/")
+		mysplit=line.split("/")
 		if len(mysplit)!=6:
 			print "Confused:",mysplit
 			continue
