@@ -1,8 +1,8 @@
 # archive_conf.py -- functionality common to archive-conf and dispatch-conf
 # Copyright 2003-2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /var/cvsroot/gentoo-src/portage/pym/dispatch_conf.py,v 1.3.2.2 2005/04/17 09:01:55 jstubbs Exp $
-cvs_id_string="$Id: dispatch_conf.py,v 1.3.2.2 2005/04/17 09:01:55 jstubbs Exp $"[5:-2]
+# $Id: dispatch_conf.py 6971 2007-06-23 13:11:08Z zmedico $
+
 
 # Library by Wayne Davison <gentoo@blorf.net>, derived from code
 # written by Jeremy Wohl (http://igmus.org)
@@ -10,7 +10,6 @@ cvs_id_string="$Id: dispatch_conf.py,v 1.3.2.2 2005/04/17 09:01:55 jstubbs Exp $
 from stat import *
 import os, sys, commands, shutil
 
-sys.path = ["/usr/lib/portage/pym"]+sys.path
 import portage
 
 RCS_BRANCH = '1.1.1'
@@ -33,7 +32,10 @@ def read_config(mandatory_opts):
 
     for key in mandatory_opts:
         if not opts.has_key(key):
-            print >> sys.stderr, 'dispatch-conf: Missing option "%s" in /etc/dispatch-conf.conf; fatal' % (key,)
+            if key == "merge":
+                opts["merge"] = "sdiff --suppress-common-lines --output=%s %s %s"
+            else:
+                print >> sys.stderr, 'dispatch-conf: Missing option "%s" in /etc/dispatch-conf.conf; fatal' % (key,)
 
     if not os.path.exists(opts['archive-dir']):
         os.mkdir(opts['archive-dir'])
