@@ -1,14 +1,13 @@
-# Copyright 1999-2006 Gentoo Foundation
+# Copyright 1999-2007 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: /var/cvsroot/gentoo-src/portage/pym/emergehelp.py,v 1.8.2.2 2005/01/16 02:35:33 carpaski Exp $
+# $Id: emergehelp.py 7348 2007-07-22 06:42:47Z zmedico $
 
 
 import os,sys
 from output import bold, turquoise, green
 
 def shorthelp():
-	print
-	print
+	print bold("emerge:")+" the other white meat (command-line interface to the Portage system)"
 	print bold("Usage:")
 	print "   "+turquoise("emerge")+" [ "+green("options")+" ] [ "+green("action")+" ] [ "+turquoise("ebuildfile")+" | "+turquoise("tbz2file")+" | "+turquoise("dependency")+" ] [ ... ]"
 	print "   "+turquoise("emerge")+" [ "+green("options")+" ] [ "+green("action")+" ] < "+turquoise("system")+" | "+turquoise("world")+" >"
@@ -16,18 +15,16 @@ def shorthelp():
 	print "   "+turquoise("emerge")+" "+turquoise("--resume")+" [ "+green("--pretend")+" | "+green("--ask")+" | "+green("--skipfirst")+" ]"
 	print "   "+turquoise("emerge")+" "+turquoise("--help")+" [ "+green("system")+" | "+green("world")+" | "+green("config")+" | "+green("--sync")+" ] "
 	print bold("Options:")+" "+green("-")+"["+green("abBcCdDefgGhikKlnNoOpqPsStuvV")+"] ["+green("--oneshot")+"] ["+green("--newuse")+"] ["+green("--noconfmem")+"]"
-	print      "                                          ["+green("--columns")+"] ["+green("--nocolor")+"] ["+green("--nospinner")+"]"
+	print      "                                          [ " + green("--color")+" < " + turquoise("y") + " | "+ turquoise("n")+" >  ] [ "+green("--columns")+" ]"
+	print      "                                     [ "+green("--reinstall ")+turquoise("changed-use")+" ] ["+green("--nospinner")+"]"
+	print "                                          [ "+green("--deep")+"  ] [" + green("--with-bdeps")+" < " + turquoise("y") + " | "+ turquoise("n")+" > ]"
 	print bold("Actions:")+" [ "+green("--clean")+" | "+green("--depclean")+" | "+green("--prune")+" | "+green("--regen")+" | "+green("--search")+" | "+green("--unmerge")+" ]"
-	print
 
 def help(myaction,myopts,havecolor=1):
-	if not havecolor:
-		nocolor()
-	if not myaction and ("--help" not in myopts):
+	if not myaction and ("--verbose" not in myopts):
 		shorthelp()
 		print
-		print "   For more help try 'emerge --help' or consult the man page."
-		print
+		print "   For more help try 'emerge --help --verbose' or consult the man page."
 	elif not myaction:
 		shorthelp()
 		print
@@ -67,10 +64,17 @@ def help(myaction,myopts,havecolor=1):
 		print "              full dependency tree from the system list and the world file,"
 		print "              then comparing it to installed packages. Packages installed, but"
 		print "              not associated with an explicit merge are listed as candidates"
-		print "              for unmerging."+turquoise(" WARNING: This can seriously affect your system by")
-		print "              "+turquoise("removing packages that may have been linked against, but due to")
-		print "              "+turquoise("changes in USE flags may no longer be part of the dep tree. Use")
-		print "              "+turquoise("caution when employing this feature.")
+		print "              for unmerging. Inexperienced users are advised to use --pretend"
+		print "              with this option in order to see a preview of which packages will"
+		print "              be uninstalled. "+turquoise(" WARNING: Removing some packages may cause")
+		print "              "+turquoise("packages which link to the removed package to stop working and")
+		print "              "+turquoise("complain about missing libraries. Rebuild the complaining package")
+		print "              "+turquoise("to fix this issue.")+" Also see --with-bdeps for behavior with"
+		print "              respect to build time dependencies that are not strictly"
+		print "              required. Depclean serves as a dependency aware version of"
+		print "              --unmerge. When given one or more atoms, it will unmerge matched"
+		print "              packages that have no reverse dependencies. Use --depclean"
+		print "              together with --verbose to show reverse dependencies."
 		print
 		print "       "+green("--info")
 		print "              Displays important portage variables that will be exported to"
@@ -80,18 +84,22 @@ def help(myaction,myopts,havecolor=1):
 		print "              run with the '--verbose' flag."
 		print
 		print "       "+green("--metadata")
-		print "              Causes portage to process all the metacache files as is normally "
-		print "              done on the tail end of an rsync update using "+bold("emerge --sync")+"."
-		print "              This processing creates the cache database that portage uses for"
-		print "              pre-parsed lookups of package data."
+		print "              Transfers metadata cache from ${PORTDIR}/metadata/cache/ to"
+		print "              /var/cache/edb/dep/ as is normally done on the tail end of an"
+		print "              rsync update using " + bold("emerge --sync") + ". This process populates the"
+		print "              cache database that portage uses for pre-parsed lookups of"
+		print "              package data.  It does not populate cache for the overlays"
+		print "              listed in PORTDIR_OVERLAY.  In order to generate cache for"
+		print "              overlays, use " + bold("--regen") + "."
 		print
 		print "       "+green("--prune")+" ("+green("-P")+" short option)"
 		print "              "+turquoise("WARNING: This action can remove important packages!")
-		print "              Removes all but the most recently installed version of a package"
+		print "              Removes all but the highest installed version of a package"
 		print "              from your system. This action doesn't verify the possible binary"
 		print "              compatibility between versions and can thus remove essential"
-		print "              dependencies from your system."
-		print "              The argument format is the same as for the "+bold("--clean")+" action."
+		print "              dependencies from your system. Use --prune together with"
+		print "              --verbose to show reverse dependencies or with --nodeps to"
+		print "              ignore all dependencies."
 		print
 		print "       "+green("--regen")
 		print "              Causes portage to check and update the dependency cache of all"
@@ -100,22 +108,31 @@ def help(myaction,myopts,havecolor=1):
 		print "              Rsync users should simply 'emerge --sync' to regenerate."
 		print
 		print "       "+green("--resume")
-		print "              Resumes the last merge operation. It can be treated just like a"
-		print "              regular emerge: --pretend and other options work alongside it."
-		print "              'emerge --resume' only returns an error on failure. When there is"
-		print "              nothing to do, it exits with a message and a success condition."
+		print "              Resumes the most recent merge list that has been aborted due to an"
+		print "              error. Please note that this operation will only return an error"
+		print "              on failure. If there is nothing for portage to do, then portage"
+		print "              will exit with a message and a success condition. A resume list"
+		print "              will persist until it has been completed in entirety or until"
+		print "              another aborted merge list replaces it. The resume history is"
+		print "              capable of storing two merge lists. After one resume list"
+		print "              completes, it is possible to invoke --resume once again in order"
+		print "              to resume an older list."
 		print
 		print "       "+green("--search")+" ("+green("-s")+" short option)"
 		print "              Searches for matches of the supplied string in the current local"
-		print "              portage tree. The search string is a regular expression."
+		print "              portage tree. By default emerge uses a case-insensitive simple "
+		print "              search, but you can enable a regular expression search by "
+		print "              prefixing the search string with %."
 		print "              Prepending the expression with a '@' will cause the category to"
 		print "              be included in the search."
 		print "              A few examples:"
-		print "              "+bold("emerge search '^kde'")
+		print "              "+bold("emerge --search libc")
+		print "                  list all packages that contain libc in their name"
+		print "              "+bold("emerge --search '%^kde'")
 		print "                  list all packages starting with kde"
-		print "              "+bold("emerge search 'gcc$'")
+		print "              "+bold("emerge --search '%gcc$'")
 		print "                  list all packages ending with gcc"
-		print "              "+bold("emerge search @^dev-java.*jdk")
+		print "              "+bold("emerge --search '%@^dev-java.*jdk'")
 		print "                  list all available Java JDKs"
 		print
 		print "       "+green("--searchdesc")+" ("+green("-S")+" short option)"
@@ -128,16 +145,19 @@ def help(myaction,myopts,havecolor=1):
 		print
 		print "       "+green("--unmerge")+" ("+green("-C")+" short option)"
 		print "              "+turquoise("WARNING: This action can remove important packages!")
-		print "              Removes all matching packages "+bold("completely")+" from"
-		print "              your system. Specify arguments using the dependency specification"
-		print "              format described in the "+bold("--clean")+" action above."
+		print "              Removes all matching packages. This does no checking of"
+		print "              dependencies, so it may remove packages necessary for the proper"
+		print "              operation of your system. Its arguments can be atoms or"
+		print "              ebuilds. For a dependency aware version of --unmerge, use"
+		print "              --depclean or --prune."
 		print
 		print "       "+green("--update")+" ("+green("-u")+" short option)"
 		print "              Updates packages to the best version available, which may not"
 		print "              always be the highest version number due to masking for testing"
 		print "              and development. This will also update direct dependencies which"
-		print "              may not what you want. In general use this option only in combi-"
-		print "              nation with the world or system target."
+		print "              may not what you want. Package atoms specified on the command line"
+		print "              are greedy, meaning that unspecific atoms may match multiple"
+		print "              installed versions of slotted packages."
 		print
 		print "       "+green("--version")+" ("+green("-V")+" short option)"
 		print "              Displays the currently installed version of portage along with"
@@ -180,6 +200,12 @@ def help(myaction,myopts,havecolor=1):
 		print "              When pretending, also display the ChangeLog entries for packages"
 		print "              that will be upgraded."
 		print
+		print "       "+green("--color") + " < " + turquoise("y") + " | "+ turquoise("n")+" >"
+		print "              Enable or disable color output. This option will override NOCOLOR"
+		print "              (see make.conf(5)) and may also be used to force color output when"
+		print "              stdout is not a tty (by default, color is disabled unless stdout"
+		print "              is a tty)."
+		print
 		print "       "+green("--columns")
 		print "              Display the pretend output in a tabular form. Versions are"
 		print "              aligned vertically."
@@ -192,11 +218,12 @@ def help(myaction,myopts,havecolor=1):
 		print "              very verbose information about the dependency and build process."
 		print
 		print "       "+green("--deep")+" ("+green("-D")+" short option)"
-		print "              When used in conjunction with --update, this flag forces emerge"
-		print "              to consider the entire dependency tree of packages, instead of"
-		print "              checking only the immediate dependencies of the packages.  As an"
-		print "              example, this catches updates in libraries that are not directly"
-		print "              listed in the dependencies of a package."
+		print "              This flag forces emerge to consider the entire dependency tree of"
+		print "              packages, instead of checking only the immediate dependencies of"
+		print "              the packages. As an example, this catches updates in libraries"
+		print "              that are not directly listed in the dependencies of a package."
+		print "              Also see --with-bdeps for behavior with respect to build time"
+		print "              dependencies that are not strictly required."
 		print 
 		print "       "+green("--emptytree")+" ("+green("-e")+" short option)"
 		print "              Virtually tweaks the tree of installed packages to contain"
@@ -230,9 +257,6 @@ def help(myaction,myopts,havecolor=1):
 		print "              Tells emerge to include installed packages where USE flags have "
 		print "              changed since installation."
 		print
-		print "       "+green("--nocolor")
-		print "              Suppresses color in the output."
-		print
 		print "       "+green("--noconfmem")
 		print "              Portage keeps track of files that have been placed into"
 		print "              CONFIG_PROTECT directories, and normally it will not merge the"
@@ -279,6 +303,12 @@ def help(myaction,myopts,havecolor=1):
 		print "              Effects vary, but the general outcome is a reduced or condensed"
 		print "              output from portage's displays."
 		print
+		print "       "+green("--reinstall ") + turquoise("changed-use")
+		print "              Tells emerge to include installed packages where USE flags have"
+		print "              changed since installation.  Unlike --newuse, this option does"
+		print "              not trigger reinstallation when flags that the user has not"
+		print "              enabled are added or removed."
+		print
 		print "       "+green("--skipfirst")
 		print "              This option is only valid in a resume situation. It removes the"
 		print "              first package in the resume list so that a merge may continue in"
@@ -309,6 +339,13 @@ def help(myaction,myopts,havecolor=1):
 		print "              Effects vary, but the general outcome is an increased or expanded"
 		print "              display of content in portage's displays."
 		print
+		print "       "+green("--with-bdeps")+" < " + turquoise("y") + " | "+ turquoise("n")+" >"
+		print "              In dependency calculations, pull in build time dependencies that"
+		print "              are not strictly required. This defaults to 'n' for installation"
+		print "              actions and 'y' for the --depclean action. This setting can be"
+		print "              added to EMERGE_DEFAULT_OPTS (see make.conf(5)) and later"
+		print "              overridden via the command line."
+		print
 	elif myaction == "sync":
 		print
 		print bold("Usage: ")+turquoise("emerge")+" "+turquoise("--sync")
