@@ -1,10 +1,18 @@
 # getbinpkg.py -- Portage binary-package helper functions
 # Copyright 2003-2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /var/cvsroot/gentoo-src/portage/pym/getbinpkg.py,v 1.12.2.2 2004/10/29 14:51:09 jstubbs Exp $
+# $Id: getbinpkg.py 7026 2007-06-25 04:50:48Z zmedico $
 
-from output import *
-import htmllib,HTMLParser,string,formatter,sys,os,xpak,time,tempfile,cPickle,base64
+if not hasattr(__builtins__, "set"):
+	from sets import Set as set
+
+from output import red, yellow, green
+import htmllib,HTMLParser,formatter,sys,os,xpak,time,tempfile,base64,urllib2
+
+try:
+	import cPickle
+except ImportError:
+	import pickle as cPickle
 
 try:
 	import ftplib
@@ -42,7 +50,7 @@ class ParseLinks(HTMLParser.HTMLParser):
 	def get_anchors_by_prefix(self,prefix):
 		newlist = []
 		for x in self.PL_anchors:
-			if (len(x) >= len(prefix)) and (x[:len(suffix)] == prefix):
+			if x.startswith(prefix):
 				if x not in newlist:
 					newlist.append(x[:])
 		return newlist
@@ -50,7 +58,7 @@ class ParseLinks(HTMLParser.HTMLParser):
 	def get_anchors_by_suffix(self,suffix):
 		newlist = []
 		for x in self.PL_anchors:
-			if (len(x) >= len(suffix)) and (x[-len(suffix):] == suffix):
+			if x.endswith(suffix):
 				if x not in newlist:
 					newlist.append(x[:])
 		return newlist
@@ -63,29 +71,35 @@ class ParseLinks(HTMLParser.HTMLParser):
 			for x in attrs:
 				if x[0] == 'href':
 					if x[1] not in self.PL_anchors:
-						self.PL_anchors.append(x[1])
+						self.PL_anchors.append(urllib2.unquote(x[1]))
 
 
 def create_conn(baseurl,conn=None):
 	"""(baseurl,conn) --- Takes a protocol://site:port/address url, and an
 	optional connection. If connection is already active, it is passed on.
 	baseurl is reduced to address and is returned in tuple (conn,address)"""
-	parts = string.split(baseurl, "://", 1)
+
+	parts = baseurl.split("://",1)
 	if len(parts) != 2:
 		raise ValueError, "Provided URL does not contain protocol identifier. '%s'" % baseurl
 	protocol,url_parts = parts
 	del parts
-	host,address = string.split(url_parts, "/", 1)
-	del url_parts
-	address = "/"+address
 
-	userpass_host = string.split(host, "@", 1)
+	url_parts = url_parts.split("/")
+	host = url_parts[0]
+	if len(url_parts) < 2:
+		address = "/"
+	else:
+		address = "/"+"/".join(url_parts[1:])
+	del url_parts
+
+	userpass_host = host.split("@",1)
 	if len(userpass_host) == 1:
 		host = userpass_host[0]
 		userpass = ["anonymous"]
 	else:
 		host = userpass_host[1]
-		userpass = string.split(userpass_host[0], ":")
+		userpass = userpass_host[0].split(":")
 	del userpass_host
 
 	if len(userpass) > 2:
@@ -103,8 +117,7 @@ def create_conn(baseurl,conn=None):
 	if username and password:
 		http_headers = {
 			"Authorization": "Basic %s" %
-			  string.replace(
-			    base64.encodestring("%s:%s" % (username, password)),
+			  base64.encodestring("%s:%s" % (username, password)).replace(
 			    "\012",
 			    ""
 			  ),
@@ -128,6 +141,15 @@ def create_conn(baseurl,conn=None):
 				conn.login(username)
 			conn.set_pasv(passive)
 			conn.set_debuglevel(0)
+		elif protocol == "sftp":
+			try:
+				import paramiko
+			except ImportError:
+				raise NotImplementedError(
+					"paramiko must be installed for sftp support")
+			t = paramiko.Transport(host)
+			t.connect(username=username, password=password)
+			conn = paramiko.SFTPClient.from_transport(t)
 		else:
 			raise NotImplementedError, "%s is not a supported protocol." % protocol
 
@@ -203,8 +225,8 @@ def make_http_request(conn, address, params={}, headers={}, dest=None):
 		if ((rc == 301) or (rc == 302)):
 			ignored_data = response.read()
 			del ignored_data
-			for x in string.split(str(response.msg), "\n"):
-				parts = string.split(x, ": ", 1)
+			for x in str(response.msg).split("\n"):
+				parts = x.split(": ",1)
 				if parts[0] == "Location":
 					if (rc == 301):
 						sys.stderr.write(red("Location has moved: ")+str(parts[1])+"\n")
@@ -294,6 +316,8 @@ def dir_get_list(baseurl,conn=None):
 			del olddir
 		else:
 			listing = conn.nlst(address)
+	elif protocol == "sftp":
+		listing = conn.listdir(address)
 	else:
 		raise TypeError, "Unknown protocol. '%s'" % protocol
 
@@ -319,6 +343,13 @@ def file_get_metadata(baseurl,conn=None, chunk_size=3000):
 		data,rc,msg = make_http_request(conn, address, params, headers)
 	elif protocol in ["ftp"]:
 		data,rc,msg = make_ftp_request(conn, address, -chunk_size)
+	elif protocol == "sftp":
+		f = conn.open(address)
+		try:
+			f.seek(-chunk_size, 2)
+			data = f.read()
+		finally:
+			f.close()
 	else:
 		raise TypeError, "Unknown protocol. '%s'" % protocol
 	
@@ -353,24 +384,19 @@ def file_get(baseurl,dest,conn=None,fcmd=None):
 	if not fcmd:
 		return file_get_lib(baseurl,dest,conn)
 
-	fcmd = string.replace(fcmd, "${DISTDIR}", dest)
-	fcmd = string.replace(fcmd, "${URI}", baseurl)
-	fcmd = string.replace(fcmd, "${FILE}", os.path.basename(baseurl))
-	mysplit = string.split(fcmd)
-	mycmd   = mysplit[0]
-	myargs  = [os.path.basename(mycmd)]+mysplit[1:]
-	mypid=os.fork()
-	if mypid == 0:
-		os.execv(mycmd,myargs)
-		sys.stderr.write("!!! Failed to spawn fetcher.\n")
-		sys.exit(1)
-	retval=os.waitpid(mypid,0)[1]
-	if (retval & 0xff) == 0:
-		retval = retval >> 8
-	else:
-		sys.stderr.write("Spawned processes caught a signal.\n")
-		sys.exit(1)
-	if retval != 0:
+	variables = {
+		"DISTDIR": dest,
+		"URI":     baseurl,
+		"FILE":    os.path.basename(baseurl)
+	}
+	import shlex, StringIO
+	from portage_util import varexpand
+	from portage_exec import spawn
+	lexer = shlex.shlex(StringIO.StringIO(fcmd), posix=True)
+	lexer.whitespace_split = True
+	myfetch = [varexpand(x, mydict=variables) for x in lexer]
+	retval = spawn(myfetch, env=os.environ.copy())
+	if retval != os.EX_OK:
 		sys.stderr.write("Fetcher exited with a failure condition.\n")
 		return 0
 	return 1
@@ -392,6 +418,25 @@ def file_get_lib(baseurl,dest,conn=None):
 		data,rc,msg = make_http_request(conn, address, params, headers, dest=dest)
 	elif protocol in ["ftp"]:
 		data,rc,msg = make_ftp_request(conn, address, dest=dest)
+	elif protocol == "sftp":
+		rc = 0
+		try:
+			f = conn.open(address)
+		except SystemExit:
+			raise
+		except Exception:
+			rc = 1
+		else:
+			try:
+				if dest:
+					bufsize = 8192
+					while True:
+						data = f.read(bufsize)
+						if not data:
+							break
+						dest.write(data)
+			finally:
+				f.close()
 	else:
 		raise TypeError, "Unknown protocol. '%s'" % protocol
 	
@@ -409,7 +454,7 @@ def dir_get_metadata(baseurl, conn=None, chunk_size=3000, verbose=1, usingcache=
 	else:
 		keepconnection = 1
 
-	if makepickle == None:
+	if makepickle is None:
 		makepickle = "/var/cache/edb/metadata.idx.most_recent"
 
 	conn,protocol,address,params,headers = create_conn(baseurl, conn)
@@ -421,9 +466,7 @@ def dir_get_metadata(baseurl, conn=None, chunk_size=3000, verbose=1, usingcache=
 		metadata = cPickle.load(metadatafile)
 		sys.stderr.write("Loaded metadata pickle.\n")
 		metadatafile.close()
-	except SystemExit, e:
-		raise
-	except:
+	except (cPickle.UnpicklingError, OSError, IOError, EOFError):
 		metadata = {}
 	if not metadata.has_key(baseurl):
 		metadata[baseurl]={}
@@ -502,20 +545,42 @@ def dir_get_metadata(baseurl, conn=None, chunk_size=3000, verbose=1, usingcache=
 			break
 	# We may have metadata... now we run through the tbz2 list and check.
 	sys.stderr.write(yellow("cache miss: 'x'")+" --- "+green("cache hit: 'o'")+"\n")
+	binpkg_filenames = set()
 	for x in tbz2list:
 		x = os.path.basename(x)
-		if ((not metadata[baseurl]["data"].has_key(x)) or \
-		    (x not in metadata[baseurl]["data"].keys())):
+		binpkg_filenames.add(x)
+		if x not in metadata[baseurl]["data"]:
 			sys.stderr.write(yellow("x"))
 			metadata[baseurl]["modified"] = 1
-			myid = file_get_metadata(baseurl+"/"+x, conn, chunk_size)
-		
-			if myid[0]:
+			myid = None
+			for retry in xrange(3):
+				try:
+					myid = file_get_metadata(
+						"/".join((baseurl.rstrip("/"), x.lstrip("/"))),
+						conn, chunk_size)
+					break
+				except httplib.BadStatusLine:
+					# Sometimes this error is thrown from conn.getresponse() in
+					# make_http_request().  The docstring for this error in
+					# httplib.py says "Presumably, the server closed the
+					# connection before sending a valid response".
+					conn, protocol, address, params, headers = create_conn(
+						baseurl)
+
+			if myid and myid[0]:
 				metadata[baseurl]["data"][x] = make_metadata_dict(myid)
 			elif verbose:
 				sys.stderr.write(red("!!! Failed to retrieve metadata on: ")+str(x)+"\n")
 		else:
 			sys.stderr.write(green("o"))
+	# Cleanse stale cache for files that don't exist on the server anymore.
+	stale_cache = set(metadata[baseurl]["data"]).difference(binpkg_filenames)
+	if stale_cache:
+		for x in stale_cache:
+			del metadata[baseurl]["data"][x]
+		metadata[baseurl]["modified"] = 1
+	del stale_cache
+	del binpkg_filenames
 	sys.stderr.write("\n")
 	
 	try:
