@@ -1,12 +1,16 @@
 # portage_data.py -- Calculated/Discovered Data Values
 # Copyright 1998-2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: /var/cvsroot/gentoo-src/portage/pym/portage_data.py,v 1.5.2.2 2005/02/26 11:22:38 carpaski Exp $
+# $Id: portage_data.py 6147 2007-03-03 10:39:35Z zmedico $
 
+if not hasattr(__builtins__, "set"):
+	from sets import Set as set
 
-import os,pwd,grp
+import os, sys, pwd, grp
 from portage_util import writemsg
 from output import green,red
+from output import create_color_func
+bad = create_color_func("BAD")
 
 ostype=os.uname()[0]
 
@@ -42,6 +46,20 @@ if not lchown:
 
 os.environ["USERLAND"]=userland
 
+def portage_group_warning():
+	warn_prefix = bad("*** WARNING ***  ")
+	mylines = [
+		"For security reasons, only system administrators should be",
+		"allowed in the portage group.  Untrusted users or processes",
+		"can potentially exploit the portage group for attacks such as",
+		"local privilege escalation."
+	]
+	for x in mylines:
+		writemsg(warn_prefix, noiselevel=-1)
+		writemsg(x, noiselevel=-1)
+		writemsg("\n", noiselevel=-1)
+	writemsg("\n", noiselevel=-1)
+
 # Portage has 3 security levels that depend on the uid and gid of the main
 # process and are assigned according to the following table:
 #
@@ -89,10 +107,20 @@ except KeyError:
 	writemsg(green("         portage:x:250:250:portage:/var/tmp/portage:/bin/false\n"))
 	writemsg(green("         portage::250:portage\n"))
 	writemsg("\n")
+	portage_group_warning()
 
-if (uid!=0) and (portage_gid not in os.getgroups()):
-	writemsg("\n")
-	writemsg(red("*** You are not in the portage group. You may experience cache problems\n"))
-	writemsg(red("*** due to permissions preventing the creation of the on-disk cache.\n"))
-	writemsg(red("*** Please add this user to the portage group if you wish to use portage.\n"))
-	writemsg("\n")
+userpriv_groups = [portage_gid]
+if secpass >= 2:
+	# Get a list of group IDs for the portage user.  Do not use grp.getgrall()
+	# since it is known to trigger spurious SIGPIPE problems with nss_ldap.
+	from commands import getstatusoutput
+	mystatus, myoutput = getstatusoutput("id -G portage")
+	if mystatus == os.EX_OK:
+		for x in myoutput.split():
+			try:
+				userpriv_groups.append(int(x))
+			except ValueError:
+				pass
+			del x
+		userpriv_groups = list(set(userpriv_groups))
+	del getstatusoutput, mystatus, myoutput
