@@ -1,13 +1,13 @@
 # Copyright 2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: portage_util.py 4400 2006-09-04 17:59:09Z zmedico $
+# $Id: portage_util.py 6931 2007-06-22 02:09:41Z zmedico $
 
 from portage_exception import PortageException, FileNotFound, \
        OperationNotPermitted, PermissionDenied, ReadOnlyFileSystem
 import portage_exception
 from portage_dep import isvalidatom
 
-import sys,string,shlex,os,errno
+import os, errno, shlex, stat, string, sys
 try:
 	import cPickle
 except ImportError:
@@ -32,6 +32,11 @@ def writemsg_stdout(mystr,noiselevel=0):
 	writemsg(mystr, noiselevel=noiselevel, fd=sys.stdout)
 
 def normalize_path(mypath):
+	""" 
+	os.path.normpath("//foo") returns "//foo" instead of "/foo"
+	We dislike this behavior so we create our own normpath func
+	to fix it.
+	"""
 	if mypath.startswith(os.path.sep):
 		# posixpath.normpath collapses 3 or more leading slashes to just 1.
 		return os.path.normpath(2*os.path.sep + mypath)
@@ -47,16 +52,16 @@ def grabfile(myfilename, compat_level=0, recursive=0):
 	for x in mylines:
 		#the split/join thing removes leading and trailing whitespace, and converts any whitespace in the line
 		#into single spaces.
-		myline=string.join(string.split(x))
+		myline=" ".join(x.split())
 		if not len(myline):
 			continue
 		if myline[0]=="#":
 			# Check if we have a compat-level string. BC-integration data.
 			# '##COMPAT==>N<==' 'some string attached to it'
-			mylinetest = string.split(myline, "<==", 1)
+			mylinetest = myline.split("<==",1)
 			if len(mylinetest) == 2:
 				myline_potential = mylinetest[1]
-				mylinetest = string.split(mylinetest[0],"##COMPAT==>")
+				mylinetest = mylinetest[0].split("##COMPAT==>")
 				if len(mylinetest) == 2:
 					if compat_level >= int(mylinetest[1]):
 						# It's a compat line, and the key matches.
@@ -71,21 +76,50 @@ def map_dictlist_vals(func,myDict):
 	"""Performs a function on each value of each key in a dictlist.
 	Returns a new dictlist."""
 	new_dl = {}
-	for key in myDict.keys():
+	for key in myDict:
 		new_dl[key] = []
 		new_dl[key] = map(func,myDict[key])
 	return new_dl
 
 def stack_dictlist(original_dicts, incremental=0, incrementals=[], ignore_none=0):
-	"""Stacks an array of dict-types into one array. Optionally merging or
+	"""
+	Stacks an array of dict-types into one array. Optionally merging or
 	overwriting matching key/value pairs for the dict[key]->list.
-	Returns a single dict. Higher index in lists is preferenced."""
+	Returns a single dict. Higher index in lists is preferenced.
+	
+	Example usage:
+	   >>> from portage_util import stack_dictlist
+		>>> print stack_dictlist( [{'a':'b'},{'x':'y'}])
+		>>> {'a':'b','x':'y'}
+		>>> print stack_dictlist( [{'a':'b'},{'a':'c'}], incremental = True )
+		>>> {'a':['b','c'] }
+		>>> a = {'KEYWORDS':['x86','alpha']}
+		>>> b = {'KEYWORDS':['-x86']}
+		>>> print stack_dictlist( [a,b] )
+		>>> { 'KEYWORDS':['x86','alpha','-x86']}
+		>>> print stack_dictlist( [a,b], incremental=True)
+		>>> { 'KEYWORDS':['alpha'] }
+		>>> print stack_dictlist( [a,b], incrementals=['KEYWORDS'])
+		>>> { 'KEYWORDS':['alpha'] }
+	
+	@param original_dicts a list of (dictionary objects or None)
+	@type list
+	@param incremental True or false depending on whether new keys should overwrite
+	   keys which already exist.
+	@type boolean
+	@param incrementals A list of items that should be incremental (-foo removes foo from
+	   the returned dict).
+	@type list
+	@param ignore_none Appears to be ignored, but probably was used long long ago.
+	@type boolean
+	
+	"""
 	final_dict = {}
 	for mydict in original_dicts:
 		if mydict is None:
 			continue
-		for y in mydict.keys():
-			if not final_dict.has_key(y):
+		for y in mydict:
+			if not y in final_dict:
 				final_dict[y] = []
 			
 			for thing in mydict[y]:
@@ -102,7 +136,7 @@ def stack_dictlist(original_dicts, incremental=0, incrementals=[], ignore_none=0
 							continue
 					if thing not in final_dict[y]:
 						final_dict[y].append(thing)
-			if final_dict.has_key(y) and not final_dict[y]:
+			if y in final_dict and not final_dict[y]:
 				del final_dict[y]
 	return final_dict
 
@@ -125,7 +159,7 @@ def stack_dicts(dicts, incremental=0, incrementals=[], ignore_none=0):
 					final_dict[y] += " "+mydict[y][:]
 				else:
 					final_dict[y]  = mydict[y][:]
-			mydict[y] = string.join(mydict[y].split()) # Remove extra spaces.
+			mydict[y] = " ".join(mydict[y].split()) # Remove extra spaces.
 	return final_dict
 
 def stack_lists(lists, incremental=1):
@@ -137,30 +171,57 @@ def stack_lists(lists, incremental=1):
 	new_list = {}
 	for x in lists:
 		for y in filter(None, x):
-			if incremental and y.startswith("-"):
-				if y[1:] in new_list:
-					del new_list[y[1:]]
+			if incremental:
+				if y == "-*":
+					new_list.clear()
+				elif y.startswith("-"):
+					new_list.pop(y[1:], None)
+				else:
+					new_list[y] = True
 			else:
 				new_list[y] = True
 	return new_list.keys()
 
-def grabdict(myfilename, juststrings=0, empty=0, recursive=0):
-	"""This function grabs the lines in a file, normalizes whitespace and returns lines in a dictionary"""
+def grabdict(myfilename, juststrings=0, empty=0, recursive=0, incremental=1):
+	"""
+	This function grabs the lines in a file, normalizes whitespace and returns lines in a dictionary
+	
+	@param myfilename: file to process
+	@type myfilename: string (path)
+	@param juststrings: only return strings
+	@type juststrings: Boolean (integer)
+	@param empty: Ignore certain lines
+	@type empty: Boolean (integer)
+	@param recursive: Recursively grab ( support for /etc/portage/package.keywords/* and friends )
+	@type recursive: Boolean (integer)
+	@param incremental: Append to the return list, don't overwrite
+	@type incremental: Boolean (integer)
+	@rtype: Dictionary
+	@returns:
+	1.  Returns the lines in a file in a dictionary, for example:
+		'sys-apps/portage x86 amd64 ppc'
+		would return
+		{ "sys-apps/portage" : [ 'x86', 'amd64', 'ppc' ]
+		the line syntax is key : [list of values]
+	"""
 	newdict={}
 	for x in grablines(myfilename, recursive):
 		#the split/join thing removes leading and trailing whitespace, and converts any whitespace in the line
 		#into single spaces.
 		if x[0] == "#":
 			continue
-		myline=string.split(x)
+		myline=x.split()
 		if len(myline) < 2 and empty == 0:
 			continue
 		if len(myline) < 1 and empty == 1:
 			continue
-		if juststrings:
-			newdict[myline[0]]=string.join(myline[1:])
+		if incremental:
+			newdict.setdefault(myline[0], []).extend(myline[1:])
 		else:
-			newdict[myline[0]]=myline[1:]
+			newdict[myline[0]] = myline[1:]
+	if juststrings:
+		for k, v in newdict.iteritems():
+			newdict[k] = " ".join(v)
 	return newdict
 
 def grabdict_package(myfilename, juststrings=0, recursive=0):
@@ -192,10 +253,12 @@ def grabfile_package(myfilename, compatlevel=0, recursive=0):
 def grablines(myfilename,recursive=0):
 	mylines=[]
 	if recursive and os.path.isdir(myfilename):
+		if myfilename in ["RCS", "CVS", "SCCS"]:
+			return mylines
 		dirlist = os.listdir(myfilename)
 		dirlist.sort()
 		for f in dirlist:
-			if not f.startswith("."):
+			if not f.startswith(".") and not f.endswith("~"):
 				mylines.extend(grablines(
 					os.path.join(myfilename, f), recursive))
 	else:
@@ -217,7 +280,7 @@ def writedict(mydict,myfilename,writekey=True):
 			for x in mydict.values():
 				myfile.write(x+"\n")
 		else:
-			for x in mydict.keys():
+			for x in mydict:
 				myfile.write("%s %s\n" % (x, " ".join(mydict[x])))
 		myfile.close()
 	except IOError:
@@ -226,7 +289,7 @@ def writedict(mydict,myfilename,writekey=True):
 		return 0
 	return 1
 
-def getconfig(mycfg,tolerant=0,allow_sourcing=False):
+def getconfig(mycfg, tolerant=0, allow_sourcing=False, expand=True):
 	mykeys={}
 	try:
 		f=open(mycfg,'r')
@@ -235,7 +298,7 @@ def getconfig(mycfg,tolerant=0,allow_sourcing=False):
 			raise
 		return None
 	try:
-		lex=shlex.shlex(f)
+		lex = shlex.shlex(f, posix=True)
 		lex.wordchars=string.digits+string.letters+"~!@#$%*_\:;?,./-+{}"     
 		lex.quotes="\"'"
 		if allow_sourcing:
@@ -244,7 +307,7 @@ def getconfig(mycfg,tolerant=0,allow_sourcing=False):
 			key=lex.get_token()
 			if key == "export":
 				key = lex.get_token()
-			if (key==''):
+			if key is None:
 				#normal end of file
 				break;
 			equ=lex.get_token()
@@ -267,7 +330,7 @@ def getconfig(mycfg,tolerant=0,allow_sourcing=False):
 				else:
 					return mykeys
 			val=lex.get_token()
-			if (val==''):
+			if val is None:
 				#unexpected end of file
 				#lex.error_leader(self.filename,lex.lineno)
 				if not tolerant:
@@ -276,7 +339,10 @@ def getconfig(mycfg,tolerant=0,allow_sourcing=False):
 					raise portage_exception.CorruptionError("ParseError: Unexpected EOF: "+str(mycfg)+": line "+str(lex.lineno))
 				else:
 					return mykeys
-			mykeys[key]=varexpand(val,mykeys)
+			if expand:
+				mykeys[key] = varexpand(val, mykeys)
+			else:
+				mykeys[key] = val
 	except SystemExit, e:
 		raise
 	except Exception, e:
@@ -285,13 +351,13 @@ def getconfig(mycfg,tolerant=0,allow_sourcing=False):
 	
 #cache expansions of constant strings
 cexpand={}
-def varexpand(mystring,mydict={}):
-	try:
-		return cexpand[" "+mystring]
-	except KeyError:
-		pass
+def varexpand(mystring, mydict={}):
+	newstring = cexpand.get(" "+mystring, None)
+	if newstring is not None:
+		return newstring
+
 	"""
-	new variable expansion code.  Removes quotes, handles \n, etc.
+	new variable expansion code.  Preserves quotes, handles \n, etc.
 	This code is used by the configfile code, as well as others (parser)
 	This would be a good bunch of code to port to C.
 	"""
@@ -307,6 +373,7 @@ def varexpand(mystring,mydict={}):
 			if (indoub):
 				newstring=newstring+"'"
 			else:
+				newstring += "'" # Quote removal is handled by shlex.
 				insing=not insing
 			pos=pos+1
 			continue
@@ -314,6 +381,7 @@ def varexpand(mystring,mydict={}):
 			if (insing):
 				newstring=newstring+'"'
 			else:
+				newstring += '"' # Quote removal is handled by shlex.
 				indoub=not indoub
 			pos=pos+1
 			continue
@@ -479,7 +547,7 @@ def unique_array(s):
 	return u
 
 def apply_permissions(filename, uid=-1, gid=-1, mode=-1, mask=-1,
-	stat_cached=None):
+	stat_cached=None, follow_links=True):
 	"""Apply user, group, and mode bits to a file if the existing bits do not
 	already match.  The default behavior is to force an exact match of mode
 	bits.  When mask=0 is specified, mode bits on the target file are allowed
@@ -492,7 +560,10 @@ def apply_permissions(filename, uid=-1, gid=-1, mode=-1, mask=-1,
 
 	if stat_cached is None:
 		try:
-			stat_cached = os.stat(filename)
+			if follow_links:
+				stat_cached = os.stat(filename)
+			else:
+				stat_cached = os.lstat(filename)
 		except OSError, oe:
 			func_call = "stat('%s')" % filename
 			if oe.errno == errno.EPERM:
@@ -507,7 +578,11 @@ def apply_permissions(filename, uid=-1, gid=-1, mode=-1, mask=-1,
 	if	(uid != -1 and uid != stat_cached.st_uid) or \
 		(gid != -1 and gid != stat_cached.st_gid):
 		try:
-			os.chown(filename, uid, gid)
+			if follow_links:
+				os.chown(filename, uid, gid)
+			else:
+				import portage_data
+				portage_data.lchown(filename, uid, gid)
 			modified = True
 		except OSError, oe:
 			func_call = "chown('%s', %i, %i)" % (filename, uid, gid)
@@ -538,6 +613,26 @@ def apply_permissions(filename, uid=-1, gid=-1, mode=-1, mask=-1,
 		if mode != st_mode:
 			new_mode = mode
 
+	# The chown system call may clear S_ISUID and S_ISGID
+	# bits, so those bits are restored if necessary.
+	if modified and new_mode == -1 and \
+		(st_mode & stat.S_ISUID or st_mode & stat.S_ISGID):
+		if mode == -1:
+			new_mode = st_mode
+		else:
+			mode = mode & 07777
+			if mask >= 0:
+				new_mode = mode | st_mode
+				new_mode = (mask ^ new_mode) & new_mode
+			else:
+				new_mode = mode
+			if not (new_mode & stat.S_ISUID or new_mode & stat.S_ISGID):
+				new_mode = -1
+
+	if not follow_links and stat.S_ISLNK(stat_cached.st_mode):
+		# Mode doesn't matter for symlinks.
+		new_mode = -1
+
 	if new_mode != -1:
 		try:
 			os.chmod(filename, new_mode)
@@ -604,7 +699,7 @@ def apply_recursive_permissions(top, uid=-1, gid=-1,
 	return all_applied
 
 def apply_secpass_permissions(filename, uid=-1, gid=-1, mode=-1, mask=-1,
-	stat_cached=None):
+	stat_cached=None, follow_links=True):
 	"""A wrapper around apply_permissions that uses secpass and simple
 	logic to apply as much of the permissions as possible without
 	generating an obviously avoidable permission exception. Despite
@@ -615,7 +710,10 @@ def apply_secpass_permissions(filename, uid=-1, gid=-1, mode=-1, mask=-1,
 
 	if stat_cached is None:
 		try:
-			stat_cached = os.stat(filename)
+			if follow_links:
+				stat_cached = os.stat(filename)
+			else:
+				stat_cached = os.lstat(filename)
 		except OSError, oe:
 			func_call = "stat('%s')" % filename
 			if oe.errno == errno.EPERM:
@@ -643,7 +741,8 @@ def apply_secpass_permissions(filename, uid=-1, gid=-1, mode=-1, mask=-1,
 			all_applied = False
 			gid = -1
 
-	apply_permissions(filename, uid=uid, gid=gid, mode=mode, mask=mask, stat_cached=stat_cached)
+	apply_permissions(filename, uid=uid, gid=gid, mode=mode, mask=mask,
+		stat_cached=stat_cached, follow_links=follow_links)
 	return all_applied
 
 class atomic_ofstream(file):
@@ -829,33 +928,70 @@ class ConfigProtect(object):
 		self.updateprotect()
 
 	def updateprotect(self):
-		#do some config file management prep
+		"""Update internal state for isprotected() calls.  Nonexistent paths
+		are ignored."""
 		self.protect = []
+		self._dirs = set()
 		for x in self.protect_list:
 			ppath = normalize_path(
-				os.path.join(self.myroot, x.lstrip(os.path.sep))) + os.path.sep
-			if os.path.isdir(ppath):
+				os.path.join(self.myroot, x.lstrip(os.path.sep)))
+			mystat = None
+			try:
+				if stat.S_ISDIR(os.stat(ppath).st_mode):
+					self._dirs.add(ppath)
 				self.protect.append(ppath)
+			except OSError:
+				# If it doesn't exist, there's no need to protect it.
+				pass
 
 		self.protectmask = []
 		for x in self.mask_list:
 			ppath = normalize_path(
-				os.path.join(self.myroot, x.lstrip(os.path.sep))) + os.path.sep
-			if os.path.isdir(ppath):
+				os.path.join(self.myroot, x.lstrip(os.path.sep)))
+			mystat = None
+			try:
+				"""Use lstat so that anything, even a broken symlink can be
+				protected."""
+				if stat.S_ISDIR(os.lstat(ppath).st_mode):
+					self._dirs.add(ppath)
 				self.protectmask.append(ppath)
-			#if it doesn't exist, silently skip it
+				"""Now use stat in case this is a symlink to a directory."""
+				if stat.S_ISDIR(os.stat(ppath).st_mode):
+					self._dirs.add(ppath)
+			except OSError:
+				# If it doesn't exist, there's no need to mask it.
+				pass
 
 	def isprotected(self, obj):
-		"""Checks if obj is in the current protect/mask directories. Returns
-		0 on unprotected/masked, and 1 on protected."""
+		"""Returns True if obj is protected, False otherwise.  The caller must
+		ensure that obj is normalized with a single leading slash.  A trailing
+		slash is optional for directories."""
 		masked = 0
 		protected = 0
+		sep = os.path.sep
 		for ppath in self.protect:
 			if len(ppath) > masked and obj.startswith(ppath):
+				if ppath in self._dirs:
+					if obj != ppath and not obj.startswith(ppath + sep):
+						# /etc/foo does not match /etc/foobaz
+						continue
+				elif obj != ppath:
+					# force exact match when CONFIG_PROTECT lists a
+					# non-directory
+					continue
 				protected = len(ppath)
 				#config file management
 				for pmpath in self.protectmask:
 					if len(pmpath) >= protected and obj.startswith(pmpath):
+						if pmpath in self._dirs:
+							if obj != pmpath and \
+								not obj.startswith(pmpath + sep):
+								# /etc/foo does not match /etc/foobaz
+								continue
+						elif obj != pmpath:
+							# force exact match when CONFIG_PROTECT_MASK lists
+							# a non-directory
+							continue
 						#skip, it's in the mask
 						masked = len(pmpath)
 		return protected > masked
@@ -873,10 +1009,6 @@ def new_protect_filename(mydest, newmd5=None):
 	prot_num = -1
 	last_pfile = ""
 
-	if len(mydest) == 0:
-		raise ValueError, "Empty path provided where a filename is required"
-	if mydest[-1] == os.path.sep: # XXX add better directory checking
-		raise ValueError, "Directory provided but this function requires a filename"
 	if not os.path.exists(mydest):
 		return mydest
 
